<?php

class AIASIST{
	
	private $options;

	private $api = 'https://aipost.ru';
		
	function __construct(){
		$this->options = get_option('_ai_assistant');
		
		if( ! isset( $this->options->token ) ){
			$this->options = new stdClass();
			$this->options->token = null;
		}
	
		add_filter('https_ssl_verify',					'__return_false');
		add_action('admin_menu',						[$this, 'menu']);
		add_action('wp_enqueue_scripts', 				[$this, 'cron'] );
		add_action('admin_enqueue_scripts', 			[$this, 'scripts'] );
		add_filter('mce_external_plugins',				[$this, 'add_button']);
		add_filter('mce_buttons', 						[$this, 'button_init'], 999);
		add_action('edit_form_after_title',				[$this, 'metabox'] );
		add_action('wp_ajax_getTask',					[$this, 'getTask']);
		add_action('wp_ajax_saveContent',				[$this, 'saveContent']);
		add_action('wp_ajax_sign',						[$this, 'sign']);
		add_action('wp_ajax_getStat',					[$this, 'getStat']);
		add_action('wp_ajax_aiassist_buy',				[$this, 'buy']);
		add_action('wp_ajax_saveStep',					[$this, 'saveStep']);
		add_action('wp_ajax_loadImage',					[$this, 'loadImage']);
		add_action('wp_ajax_saveKey',					[$this, 'saveKey']);
		add_action('wp_ajax_autoGenOptions',			[$this, 'autoGenOptions']);
		add_action('wp_ajax_stopArticlesGen',			[$this, 'stopArticlesGen']);
		add_action('wp_ajax_clearArticlesGen',			[$this, 'clearArticlesGen']);
		add_action('wp_ajax_initArticlesGen',			[$this, 'initArticlesGen']);
		add_action('wp_ajax_startArticlesGen',			[$this, 'startArticlesGen']);
		add_action('wp_ajax_aiArticlesAutoGen',			[$this, 'aiArticlesAutoGen']);
		add_action('wp_ajax_nopriv_aiArticlesAutoGen',	[$this, 'aiArticlesAutoGen']);
		
	}

	public function menu(){
		add_menu_page('AI WP Writer', 'AI WP Writer', 'level_10', 'wpai-assistant', [$this, 'options'], 'dashicons-businessperson', 777);
	}
	
	public function options(){
		if( isset( $_POST['save'] ) ){
			if( isset( $_POST['token'] ) )
				$this->activation( sanitize_text_field( $_POST['token'] ) );
		
			$this->options = new stdClass();
			
			if( isset( $_POST['token'] ) && preg_match('/^[A-Za-z0-9]{64}$/i', $_POST['token']) )
				$this->options->token = sanitize_text_field( $_POST['token'] );
			
			update_option('_ai_assistant', $this->options );
		}
		$info = $this->getInfo();
		$steps = get_option('_aiassist_generator');
		$autoGen = get_option('aiArticlesAutoGenData');
		include dirname(__FILE__) . '/tpl/options.php';
	}
	
	public function sign(){
		if( ! empty( $_POST ) ){
			@$_POST['action'] = sanitize_text_field( @$_POST['act'] );
			wp_die( $this->wpCurl( $this->api, $_POST ) );
		}
		
		wp_die('{"success":"false"}');
	}
	
	public function buy(){
		wp_die( $this->wpcurl( $this->api, [ 'token' => sanitize_text_field( $this->options->token ), 'action' => 'getPayUrl', 'type' => sanitize_text_field( $_POST['type'] ), 'out_summ' => sanitize_text_field( $_POST['out_summ'] ), 'gptkey' => sanitize_text_field( get_option('aiassist_gpt_key') ) ] ) );
	}
	
	private function activation( $token ){
		$this->wpcurl( $this->api, [ 'host' => $this->getHost(), 'action' => 'activation', 'token' => sanitize_text_field( $token ) ] );
	}
	
	private function getInfo(){
		if( $info = json_decode( $this->wpcurl( $this->api, [ 'action' => 'getInfo', 'token' => sanitize_text_field( $this->options->token ) ] ) ) )
			return $info;
		
		return false;
	}
	
	public function saveKey(){
		if( isset( $_POST['key'] ) )
			update_option('aiassist_gpt_key', sanitize_text_field( $_POST['key'] ) );
	}
	
	public function getStat(){
		wp_die( $this->wpcurl( $this->api, [ 'token' => sanitize_text_field( $this->options->token ), 'action' => 'getStat', 'host' => sanitize_text_field( $this->getHost() ), 'dateStart' => sanitize_text_field( $_REQUEST['dateStart'] ), 'dateEnd' => sanitize_text_field( $_REQUEST['dateEnd'] ) ] ) );
	}
	
	private function getHost(){
		$host = get_option('siteurl');
		
		if( $_SERVER['HTTPS'] == 'on' )
			$host = str_replace('http://', 'https://', $host);
			
		return $host;
	}

	public function aiArticlesAutoGen(){
		if( $data = get_option('aiArticlesAutoGenData') ){
			
			if( isset( $_REQUEST['dev'] ) ){
				echo "<pre>"; 
					print_r( $data ); 
				echo "</pre>"; 
			}
		
			if( ! @$data['start'] || ! @$data['articles'] )
				wp_die( json_encode( $data ) );
			
			if( $data['check'] > time() - 60 )
				wp_die( json_encode( $data ) );
			
			$data['check'] = time();
			update_option('aiArticlesAutoGenData', $data);
			
			$key = date('Ymd');
			
			if( ! isset( $data['counter'][ $key ] ) )
				$data['counter'] = [ $key => 0 ];
				
			if( ! @$data['publishInDay'] )
				$data['publishInDay'] = +100500;
				
			if( $data['counter'][ $key ] > $data['publishInDay'] )
				wp_die( json_encode( $data ) );
			
			$break = false;
			$data['publish'] = 0;
			$promts = get_option('_aiassist_generator');
			
			foreach( $data['articles'] as $k => $article ){
			
				if( $article['post_id'] )
					$data['publish']++;
				
				if( $break )
					continue;
			
				if( ! isset( $article['task_id'] ) && $data['publishInDay'] > $data['counter'][ $key ] ){
					$break = true;
					
					$args = [
								'thumb'			=> (bool) $data['thumb'], 
								'images'		=> (bool) $data['images'], 
								'textModel'		=> $data['textModel'], 
								'imageModel'	=> $data['imageModel'], 
								'promt'			=> $promts['aiassist-generation-prom'],
								'title'			=> $promts['aiassist-title-prom'],
								'description'	=> $promts['aiassist-desc-prom'],
								'keywords'		=> $article['keywords'], 
								'action'		=> 'addAutoTask', 
								'token'			=> $this->options->token, 
								'gptkey'		=> get_option('aiassist_gpt_key'),
							];
							
					$task = json_decode( $this->wpCurl($this->api, $args ) );
					
					if( $task->task_id ){
						$data['counter'][ $key ]++;
						$data['articles'][ $k ]['task_id'] = $task->task_id;
					}
				
				}
				
				if( isset( $article['task_id'] ) && ! isset( $article['post_id'] ) ){
					$break = true;
					$task = json_decode( $this->wpCurl( $this->api, [ 'action' => 'getTask', 'id' => $article['task_id'], 'host' => $this->getHost(), 'token' => $this->options->token ] ) );
					
					if( isset( $task->content ) ){
						
						$args = [
							'post_status'	=> $data['draft'] ? 'draft' : 'publish', 
							'post_title'	=> sanitize_text_field( wp_unslash( $task->header ) ), 
							'post_content'	=> wp_kses_post( wp_unslash( $task->content ) ),
							'post_category'	=> [ (int) $article['cat_id'] ],
						];
						
						if( $post_id = wp_insert_post( $args ) ){
							$data['publish']++;
							$data['articles'][ $k ]['post_id'] = $post_id;
							
							update_option('aiArticlesAutoGenData', $data);
							update_post_meta($post_id, '_title', sanitize_text_field( wp_unslash( $task->title ) ) );
							update_post_meta($post_id, '_description', sanitize_text_field( wp_unslash( $task->description ) ) );
							update_post_meta($post_id, '_yoast_wpseo_title', sanitize_text_field( wp_unslash( $task->title ) ) );
							update_post_meta($post_id, '_yoast_wpseo_metadesc', sanitize_text_field( wp_unslash( $task->description ) ) );
							update_post_meta($post_id, 'rank_math_title', sanitize_text_field( wp_unslash( $task->title ) ) );
							update_post_meta($post_id, 'rank_math_description', sanitize_text_field( wp_unslash( $task->description ) ) );
						
							if( $task->thumb ){
								if( $thumb_id = (int) $this->loadFile( $this->api .'/?action=getImage&image='. $task->thumb, $post_id ) )
									set_post_thumbnail( $post_id, $thumb_id );
							}
							
							if( $task->images ){
								$task->alts = (array) $task->alts;
								
								foreach( $task->images as $k => $src ){
									
									if( strpos($task->content, $src) === false )
										continue;
								
									if( $img_id = (int) $this->loadFile( $this->api .'/?action=getImage&image='. $src, $post_id ) ){
										$image = wp_get_attachment_image( $img_id, 'full', false, [ 'class' => 'size-full wp-image-'. $img_id .' aligncenter', 'alt' => $this->clearTitle( $task->alts[ $k ] ) .' фото', 'title' => $this->clearTitle( $task->alts[ $k ] ) ] );		
										$task->content = str_replace($src, $image, $task->content);
									}
								}
								wp_update_post( [ 'ID' => $post_id, 'post_content' => $task->content ] );
							}
							
						}
					}
				}
			}
			
			if( $data['publishInDay'] == 100500 )
				unset( $data['publishInDay'] );
			
			update_option('aiArticlesAutoGenData', $data);
			wp_die( json_encode( $data ) );
		}
	}
	
	public function autoGenOptions(){
		$data = get_option('aiArticlesAutoGenData');
		$data['draft'] = (bool) $_POST['draft'];
		$data['thumb'] = (bool) $_POST['thumb'];
		$data['images'] = (bool) $_POST['images'];
		$data['publishInDay'] = (int) $_POST['publishInDay'];
		$data['imageModel'] = sanitize_text_field( $_POST['imageModel'] );
		$data['textModel'] = sanitize_text_field( $_POST['textModel'] );
		update_option('aiArticlesAutoGenData', $data);
	}
	
	public function stopArticlesGen(){
		$data = get_option('aiArticlesAutoGenData');
		$data['start'] = false;
		update_option('aiArticlesAutoGenData', $data);
	}
	
	public function startArticlesGen(){
		$data = get_option('aiArticlesAutoGenData');
		$data['start'] = true;
		update_option('aiArticlesAutoGenData', $data);
	}
	
	public function clearArticlesGen(){
		$args = get_option('aiArticlesAutoGenData');
		$data = [ 'start' => true, 'count' => 0, 'publish' => 0, 'articles' => [] ];
			
		if( isset( $args['publishInDay'] ) )
			$data['publishInDay'] = (int) $args['publishInDay'];
			
		if( isset( $args['draft'] ) )
			$data['draft'] = (bool) $args['draft'];
			
		if( isset( $args['thumb'] ) )
			$data['thumb'] = (bool) $args['thumb'];
			
		if( isset( $args['images'] ) )
			$data['images'] = (bool) $args['images'];
			
		if( isset( $args['imageModel'] ) )
			$data['imageModel'] = sanitize_text_field( $args['imageModel'] );
			
		if( isset( $args['textModel'] ) )
			$data['textModel'] = sanitize_text_field( $args['textModel'] );
		
		update_option('aiArticlesAutoGenData', $data);
	}
	
	public function initArticlesGen(){
		if( $articles = $_POST['articles'] ){			
			$args = get_option('aiArticlesAutoGenData');
			$data = [ 'start' => true, 'count' => 0, 'publish' => 0, 'articles' => [] ];
				
			if( isset( $args['publishInDay'] ) )
				$data['publishInDay'] = (int) $args['publishInDay'];
				
			if( isset( $args['draft'] ) )
				$data['draft'] = (int) $args['draft'];
				
			if( isset( $args['thumb'] ) )
				$data['thumb'] = (int) $args['thumb'];
				
			if( isset( $args['images'] ) )
				$data['images'] = (int) $args['images'];
			
			if( ! empty( $args['articles'] ) )
				$data['articles'] = $args['articles'];
				
			if( ! empty( $args['imageModel'] ) )
				$data['imageModel'] = $args['imageModel'];
				
			if( ! empty( $args['textModel'] ) )
				$data['textModel'] = $args['textModel'];
			
			foreach( $articles as $cat_id => $article ){
				if( $items = explode("\n", sanitize_textarea_field( $article ) ) ){
					foreach( $items as $item )
						$data['articles'][] = [ 'keywords' => $item, 'cat_id' => $cat_id ];
				}
			}
			$data['count'] = count( $data['articles'] );
			update_option('aiArticlesAutoGenData', $data);
			
			if( ! ( $steps = get_option('_aiassist_generator') ) )
				$steps = [];
			
			if( isset( $_POST['artPromt'] ) )
				$steps['aiassist-generation-prom'] = sanitize_text_field( $_POST['artPromt'] );
			
			if( isset( $_POST['titlePromt'] ) )
				$steps['aiassist-title-prom'] = sanitize_text_field( $_POST['titlePromt'] );
			
			if( isset( $_POST['descPromt'] ) )
				$steps['aiassist-desc-prom'] = sanitize_text_field( $_POST['descPromt'] );
			
			update_option('_aiassist_generator', $steps);
			
			wp_die( json_encode( $data ) );
		}
		wp_die('{"success":"false"}');
	}
	
	public function saveStep(){
		if( isset( $_POST['val'] ) && isset( $_POST['act'] ) ){
			$val = wp_kses_post( wp_unslash( $_POST['val'] ) );
			$act = sanitize_text_field( $_POST['act'] );
			
			if( ! ( $data = get_option('_aiassist_generator') ) )
				$data = [];
				
			$data[ $act ] = $val;
			
			update_option('_aiassist_generator', $data);
			wp_die('{"success":"true"}');
		}
		wp_die('{"success":"false"}');
	}

	public function resetStep(){
		if( ! ( $steps = get_option('_aiassist_generator') ) )
			$steps = [];
		
		$data = [
			'aiassist-article-prom'		=> @$steps['aiassist-article-prom'],
			'aiassist-theme-prom'		=> @$steps['aiassist-theme-prom'],
			'aiassist-structure-prom'	=> @$steps['aiassist-structure-prom'],
			'aiassist-content-prom'		=> @$steps['aiassist-content-prom'],
			'aiassist-title-prom'		=> @$steps['aiassist-title-prom'],
			'aiassist-desc-prom'		=> @$steps['aiassist-desc-prom'],
			'aiassist-prom-regenerate'	=> @$steps['aiassist-prom-regenerate'],
			'aiassist-generation-prom'	=> @$steps['aiassist-generation-prom'],
		];
		
		update_option('_aiassist_generator', $data);
	}

	public function loadImage(){
		if( $post_id = (int) $_POST['post_id'] ){
			
			$url = sanitize_url( $_POST['image']['src'] );
			$title = sanitize_text_field( $_POST['image']['title'] );
		
			if( $url && $title ){
				if( $img_id = (int) $this->loadFile( $url, $post_id ) ){
					$image = wp_get_attachment_image( $img_id, 'full', false, [ 'class' => 'size-full wp-image-'. $img_id .' aligncenter', 'alt' => $this->clearTitle( $title ) .' фото', 'title' => $this->clearTitle( $title ) ] );		
					wp_die( json_encode( [ 'id' => $img_id, 'url' => wp_get_attachment_url( $img_id ), 'image' => $image ] ) );
				}
			}
			
		}
		wp_die('{"success":"false"}');
	}
	
	public function saveContent(){
		if( $post_id = (int) $_POST['post_id'] ){
			
			if( ! trim( $_POST['header'] ) ){
				if( preg_match('/<h1[^>]*>([^<]+)<\/h1>/isu', $_POST['content'], $header) ){
					$_POST['header'] = $header[1];
					$_POST['content'] = str_replace($header[0], '', $_POST['content']);
				} else {			
					if( preg_match('/\# ([а-яА-Яa-zA-Z0-9 :_-]+)/isu', $_POST['content'], $header) ){
						$_POST['header'] = $header[1];
						$_POST['content'] = str_replace($header[0], '', $_POST['content']);
					}
				}
			}
			
			if( $_POST['thumbnail'] ){
				if( $thumb_id = (int) $this->loadFile( $_POST['thumbnail'], $post_id ) )
					set_post_thumbnail( $post_id, $thumb_id );
			}

			$args = [
						'ID' => (int) $post_id, 
						'post_status' => 'draft', 
						'post_title' => sanitize_text_field( wp_unslash( $_POST['header'] ) ), 
						'post_content' => wp_kses_post( wp_unslash( $_POST['content'] ) ) 
					];
		
			if( $post_id = wp_update_post( $args ) ){
				update_post_meta($post_id, '_title', sanitize_text_field( wp_unslash( $_POST['title'] ) ) );
				update_post_meta($post_id, '_description', sanitize_text_field( wp_unslash( $_POST['desc'] ) ) );
				
				update_post_meta($post_id, '_yoast_wpseo_title', sanitize_text_field( wp_unslash( $_POST['title'] ) ) );
				update_post_meta($post_id, '_yoast_wpseo_metadesc', sanitize_text_field( wp_unslash( $_POST['desc'] ) ) );
				
				update_post_meta($post_id, 'rank_math_title', sanitize_text_field( wp_unslash( $_POST['title'] ) ) );
				update_post_meta($post_id, 'rank_math_description', sanitize_text_field( wp_unslash( $_POST['desc'] ) ) );
				
				$this->resetStep();
				wp_die('{"id":"'. $post_id .'"}');
			}
		}
		wp_die('{"success":"false"}');
	}

	private function clearTitle( $title ){
		return preg_replace('/^[0-9]*.? ?/i', '', $title);
	}
	
	private function loadFile( $url, $post_id ){
		$tmp = download_url( $url, 300 );
		
		if( preg_match('/image=([^\.]+).(png|jpg)/i', $url, $matches ) ){
			$file = [ 'name' => $matches[1] .'.'. $matches[2], 'tmp_name' => $tmp ];
			$id = media_handle_sideload( $file, $post_id );
			
			if( is_wp_error( $id ) ){
				@unlink($file['tmp_name']);
				return false;
			}
			@unlink( $file['tmp_name'] );
		
			return $id;
		}
		return false;
	}

	public function getTask(){
		if( ( $id = (int) @$_POST['id'] ) )
			wp_die( $this->wpCurl( $this->api, [ 'id' => $id, 'host' => sanitize_text_field( $this->getHost() ), 'action' => 'getTask', 'token' => sanitize_text_field( $this->options->token ) ] ) );
		
		wp_die('{"success":"false"}');
	}

	public function cron(){
		wp_enqueue_script('aiassist-cron', plugin_dir_url( __FILE__ ) .'assets/js/cron.js?t='. time(), false, false, false );
		wp_localize_script('aiassist-cron', 'aiassist', [ 'ajaxurl' => admin_url('admin-ajax.php') ] );
	}
	
	public function scripts(){
		wp_enqueue_style('aiassist', plugin_dir_url( __FILE__ ) .'assets/css/style.css?t='. time(), false, '1.0', 'all');
		
		wp_enqueue_script('google-charts', plugin_dir_url( __FILE__ ) .'assets/libs/charts.js', [ 'jquery' ], false, false );
		wp_enqueue_script('aiassist', plugin_dir_url( __FILE__ ) .'assets/js/app.js?t='. time(), [ 'jquery', 'wp-data' ], false, false );
		wp_enqueue_script('aiassist-image-block', plugin_dir_url( __FILE__ ) .'assets/js/image-block.js?t='. time(), [ 'wp-blocks', 'wp-element', 'jquery', 'aiassist' ], false, false );
		
		wp_localize_script('aiassist', 'aiassist', [ 
			'ajaxurl'	=> admin_url('admin-ajax.php'), 
			'apiurl'	=> $this->api,
			'token'		=> $this->options->token,
			'gptkey'	=> get_option('aiassist_gpt_key')
		] );
	}
	
	public function button_init( $buttons ){
		$buttons[] = 'AIASSIST';
		$buttons[] = 'ai_image';
		return $buttons;
	}
	
	public function add_button( $data ){
		if( current_user_can('edit_posts') && current_user_can('edit_pages') ){
			$data['button'] = plugin_dir_url( __FILE__ ) .'assets/js/button.js?t='. time();
			$data['button2'] = plugin_dir_url( __FILE__ ) .'assets/js/image-tiny-mce.js?t='. time();
		}
		
		return $data;
	}
	
	public function metabox(){
		add_meta_box('ai_assistant', 'AI WP Writer', function(){
			$info = $this->getInfo();
			$steps = get_option('_aiassist_generator');
			include __DIR__ .'/tpl/workspace.php';
		}, array_values( get_post_types() ), 'normal', 'high' );
	}
	
	private function wpCurl( $url, $args = [] ){
		if( ! empty( $args ) )
			$args = [ 'body' => $args, 'timeout' => 300 ];
		
		$data = (array) wp_remote_request( $url, $args );
		
		return @$data['body'];
	}
	
}

?>